@@ -25,31 +25,16 @@ class AuthorWithDefaultHeight(models.Model):
         apps = new_apps
 
 
-class AuthorWithM2M(models.Model):
+class AuthorWithEvenLongerName(models.Model):
     name = models.CharField(max_length=255)
-
-    class Meta:
-        apps = new_apps
-
-
-class AuthorWithM2MThrough(models.Model):
-    name = models.CharField(max_length=255)
-    tags = models.ManyToManyField("schema.TagM2MTest", related_name="authors", through="AuthorTag")
-
-    class Meta:
-        apps = new_apps
-
-
-class AuthorTag(models.Model):
-    author = models.ForeignKey("schema.AuthorWithM2MThrough")
-    tag = models.ForeignKey("schema.TagM2MTest")
+    height = models.PositiveIntegerField(null=True, blank=True)
 
     class Meta:
         apps = new_apps
 
 
 class Book(models.Model):
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
     # tags = models.ManyToManyField("Tag", related_name="books")
@@ -59,7 +44,7 @@ class Book(models.Model):
 
 
 class BookWeak(models.Model):
-    author = models.ForeignKey(Author, db_constraint=False)
+    author = models.ForeignKey(Author, models.CASCADE, db_constraint=False)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
 
@@ -67,8 +52,18 @@ class BookWeak(models.Model):
         apps = new_apps
 
 
+class BookWithLongName(models.Model):
+    author_foreign_key_with_really_long_field_name = models.ForeignKey(
+        AuthorWithEvenLongerName,
+        models.CASCADE,
+    )
+
+    class Meta:
+        apps = new_apps
+
+
 class BookWithO2O(models.Model):
-    author = models.OneToOneField(Author)
+    author = models.OneToOneField(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
 
@@ -77,33 +72,8 @@ class BookWithO2O(models.Model):
         db_table = "schema_book"
 
 
-class BookWithM2M(models.Model):
-    author = models.ForeignKey(Author)
-    title = models.CharField(max_length=100, db_index=True)
-    pub_date = models.DateTimeField()
-    tags = models.ManyToManyField("TagM2MTest", related_name="books")
-
-    class Meta:
-        apps = new_apps
-
-
-class TagThrough(models.Model):
-    book = models.ForeignKey("schema.BookWithM2MThrough")
-    tag = models.ForeignKey("schema.TagM2MTest")
-
-    class Meta:
-        apps = new_apps
-
-
-class BookWithM2MThrough(models.Model):
-    tags = models.ManyToManyField("TagM2MTest", related_name="books", through=TagThrough)
-
-    class Meta:
-        apps = new_apps
-
-
 class BookWithSlug(models.Model):
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
     pub_date = models.DateTimeField()
     slug = models.CharField(max_length=20, unique=True)
@@ -113,15 +83,48 @@ class BookWithSlug(models.Model):
         db_table = "schema_book"
 
 
-class Tag(models.Model):
-    title = models.CharField(max_length=255)
-    slug = models.SlugField(unique=True)
+class BookWithoutAuthor(models.Model):
+    title = models.CharField(max_length=100, db_index=True)
+    pub_date = models.DateTimeField()
+
+    class Meta:
+        apps = new_apps
+        db_table = "schema_book"
+
+
+class BookForeignObj(models.Model):
+    title = models.CharField(max_length=100, db_index=True)
+    author_id = models.IntegerField()
 
     class Meta:
         apps = new_apps
 
 
-class TagM2MTest(models.Model):
+class IntegerPK(models.Model):
+    i = models.IntegerField(primary_key=True)
+    j = models.IntegerField(unique=True)
+
+    class Meta:
+        apps = new_apps
+        db_table = "INTEGERPK"  # uppercase to ensure proper quoting
+
+
+class Note(models.Model):
+    info = models.TextField()
+
+    class Meta:
+        apps = new_apps
+
+
+class NoteRename(models.Model):
+    detail_info = models.TextField()
+
+    class Meta:
+        apps = new_apps
+        db_table = "schema_note"
+
+
+class Tag(models.Model):
     title = models.CharField(max_length=255)
     slug = models.SlugField(unique=True)
 
@@ -138,6 +141,14 @@ class TagIndexed(models.Model):
         index_together = [["slug", "title"]]
 
 
+class TagM2MTest(models.Model):
+    title = models.CharField(max_length=255)
+    slug = models.SlugField(unique=True)
+
+    class Meta:
+        apps = new_apps
+
+
 class TagUniqueRename(models.Model):
     title = models.CharField(max_length=255)
     slug2 = models.SlugField(unique=True)
@@ -147,30 +158,6 @@ class TagUniqueRename(models.Model):
         db_table = "schema_tag"
 
 
-class UniqueTest(models.Model):
-    year = models.IntegerField()
-    slug = models.SlugField(unique=False)
-
-    class Meta:
-        apps = new_apps
-        unique_together = ["year", "slug"]
-
-
-class AuthorWithEvenLongerName(models.Model):
-    name = models.CharField(max_length=255)
-    height = models.PositiveIntegerField(null=True, blank=True)
-
-    class Meta:
-        apps = new_apps
-
-
-class BookWithLongName(models.Model):
-    author_foreign_key_with_really_long_field_name = models.ForeignKey(AuthorWithEvenLongerName)
-
-    class Meta:
-        apps = new_apps
-
-
 # Based on tests/reserved_names/models.py
 @python_2_unicode_compatible
 class Thing(models.Model):
@@ -183,5 +170,10 @@ class Thing(models.Model):
         return self.when
 
 
-class Note(models.Model):
-    info = models.TextField()
+class UniqueTest(models.Model):
+    year = models.IntegerField()
+    slug = models.SlugField(unique=False)
+
+    class Meta:
+        apps = new_apps
+        unique_together = ["year", "slug"]
