@@ -1,33 +1,16 @@
-from datetime import datetime
+# -*- encoding: utf-8 -*-
+from __future__ import unicode_literals
+
 import sys
 import unittest
+from datetime import datetime
 
+from django.utils import http, six
 from django.utils.datastructures import MultiValueDict
-from django.utils import http
-from django.utils import six
 
 
 class TestUtilsHttp(unittest.TestCase):
 
-    def test_same_origin_true(self):
-        # Identical
-        self.assertTrue(http.same_origin('http://foo.com/', 'http://foo.com/'))
-        # One with trailing slash - see #15617
-        self.assertTrue(http.same_origin('http://foo.com', 'http://foo.com/'))
-        self.assertTrue(http.same_origin('http://foo.com/', 'http://foo.com'))
-        # With port
-        self.assertTrue(http.same_origin('https://foo.com:8000', 'https://foo.com:8000/'))
-
-    def test_same_origin_false(self):
-        # Different scheme
-        self.assertFalse(http.same_origin('http://foo.com', 'https://foo.com'))
-        # Different host
-        self.assertFalse(http.same_origin('http://foo.com', 'http://goo.com'))
-        # Different host again
-        self.assertFalse(http.same_origin('http://foo.com', 'http://foo.com.evil.com'))
-        # Different port
-        self.assertFalse(http.same_origin('http://foo.com:8000', 'http://foo.com:8001'))
-
     def test_urlencode(self):
         # 2-tuples (the norm)
         result = http.urlencode((('a', 1), ('b', 2), ('c', 3)))
@@ -45,7 +28,7 @@ class TestUtilsHttp(unittest.TestCase):
             'c=3&a=1&b=2',
             'c=3&b=2&a=1'
         ]
-        self.assertTrue(result in acceptable_results)
+        self.assertIn(result, acceptable_results)
         result = http.urlencode({'a': [1, 2]}, doseq=False)
         self.assertEqual(result, 'a=%5B%271%27%2C+%272%27%5D')
         result = http.urlencode({'a': [1, 2]}, doseq=True)
@@ -63,7 +46,7 @@ class TestUtilsHttp(unittest.TestCase):
             'name=Adrian&name=Simon&position=Developer',
             'position=Developer&name=Adrian&name=Simon'
         ]
-        self.assertTrue(result in acceptable_results)
+        self.assertIn(result, acceptable_results)
 
     def test_base36(self):
         # reciprocity works
@@ -93,7 +76,7 @@ class TestUtilsHttp(unittest.TestCase):
         for bad_url in ('http://example.com',
                         'http:///example.com',
                         'https://example.com',
-                        'ftp://exampel.com',
+                        'ftp://example.com',
                         r'\\example.com',
                         r'\\\example.com',
                         r'/\\/example.com',
@@ -107,18 +90,91 @@ class TestUtilsHttp(unittest.TestCase):
                         'http:/\//example.com',
                         'http:\/example.com',
                         'http:/\example.com',
-                        'javascript:alert("XSS")'):
+                        'javascript:alert("XSS")',
+                        '\njavascript:alert(x)',
+                        '\x08//example.com',
+                        r'http://otherserver\@example.com',
+                        r'http:\\testserver\@example.com',
+                        r'http://testserver\me:pass@example.com',
+                        r'http://testserver\@example.com',
+                        r'http:\\testserver\confirm\me@example.com',
+                        '\n'):
             self.assertFalse(http.is_safe_url(bad_url, host='testserver'), "%s should be blocked" % bad_url)
         for good_url in ('/view/?param=http://example.com',
                      '/view/?param=https://example.com',
-                     '/view?param=ftp://exampel.com',
+                     '/view?param=ftp://example.com',
                      'view/?param=//example.com',
                      'https://testserver/',
                      'HTTPS://testserver/',
                      '//testserver/',
+                     'http://testserver/confirm?email=me@example.com',
                      '/url%20with%20spaces/'):
             self.assertTrue(http.is_safe_url(good_url, host='testserver'), "%s should be allowed" % good_url)
 
+        if six.PY2:
+            # Check binary URLs, regression tests for #26308
+            self.assertTrue(
+                http.is_safe_url(b'https://testserver/', host='testserver'),
+                "binary URLs should be allowed on Python 2"
+            )
+            self.assertFalse(http.is_safe_url(b'\x08//example.com', host='testserver'))
+            self.assertTrue(http.is_safe_url('àview/'.encode('utf-8'), host='testserver'))
+            self.assertFalse(http.is_safe_url('àview'.encode('latin-1'), host='testserver'))
+
+        # Valid basic auth credentials are allowed.
+        self.assertTrue(http.is_safe_url(r'http://user:pass@testserver/', host='user:pass@testserver'))
+        # A path without host is allowed.
+        self.assertTrue(http.is_safe_url('/confirm/me@example.com'))
+        # Basic auth without host is not allowed.
+        self.assertFalse(http.is_safe_url(r'http://testserver\@example.com'))
+
+    def test_urlsafe_base64_roundtrip(self):
+        bytestring = b'foo'
+        encoded = http.urlsafe_base64_encode(bytestring)
+        decoded = http.urlsafe_base64_decode(encoded)
+        self.assertEqual(bytestring, decoded)
+
+    def test_urlquote(self):
+        self.assertEqual(http.urlquote('Paris & Orl\xe9ans'),
+            'Paris%20%26%20Orl%C3%A9ans')
+        self.assertEqual(http.urlquote('Paris & Orl\xe9ans', safe="&"),
+            'Paris%20&%20Orl%C3%A9ans')
+        self.assertEqual(
+            http.urlunquote('Paris%20%26%20Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(
+            http.urlunquote('Paris%20&%20Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(http.urlquote_plus('Paris & Orl\xe9ans'),
+            'Paris+%26+Orl%C3%A9ans')
+        self.assertEqual(http.urlquote_plus('Paris & Orl\xe9ans', safe="&"),
+            'Paris+&+Orl%C3%A9ans')
+        self.assertEqual(
+            http.urlunquote_plus('Paris+%26+Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+        self.assertEqual(
+            http.urlunquote_plus('Paris+&+Orl%C3%A9ans'),
+            'Paris & Orl\xe9ans')
+
+    def test_is_same_domain_good(self):
+        for pair in (
+            ('example.com', 'example.com'),
+            ('example.com', '.example.com'),
+            ('foo.example.com', '.example.com'),
+            ('example.com:8888', 'example.com:8888'),
+            ('example.com:8888', '.example.com:8888'),
+            ('foo.example.com:8888', '.example.com:8888'),
+        ):
+            self.assertTrue(http.is_same_domain(*pair))
+
+    def test_is_same_domain_bad(self):
+        for pair in (
+            ('example2.com', 'example.com'),
+            ('foo.example.com', 'example.com'),
+            ('example.com:9999', 'example.com:8888'),
+        ):
+            self.assertFalse(http.is_same_domain(*pair))
+
 
 class ETagProcessingTests(unittest.TestCase):
     def test_parsing(self):
@@ -126,11 +182,21 @@ class ETagProcessingTests(unittest.TestCase):
         self.assertEqual(etags, ['', 'etag', 'e"t"ag', r'e\tag', 'weak'])
 
     def test_quoting(self):
-        quoted_etag = http.quote_etag(r'e\t"ag')
+        original_etag = r'e\t"ag'
+        quoted_etag = http.quote_etag(original_etag)
         self.assertEqual(quoted_etag, r'"e\\t\"ag"')
+        self.assertEqual(http.unquote_etag(quoted_etag), original_etag)
 
 
 class HttpDateProcessingTests(unittest.TestCase):
+    def test_http_date(self):
+        t = 1167616461.0
+        self.assertEqual(http.http_date(t), 'Mon, 01 Jan 2007 01:54:21 GMT')
+
+    def test_cookie_date(self):
+        t = 1167616461.0
+        self.assertEqual(http.cookie_date(t), 'Mon, 01-Jan-2007 01:54:21 GMT')
+
     def test_parsing_rfc1123(self):
         parsed = http.parse_http_date('Sun, 06 Nov 1994 08:49:37 GMT')
         self.assertEqual(datetime.utcfromtimestamp(parsed),
